<?php
/***************************************
** Filename.......: class.smtp.inc
** Project........: SMTP Class
** Version........: 1.00
** Last Modified..: 30 September 2001
***************************************/

	define('SMTP_STATUS_NOT_CONNECTED', 1, TRUE);
	define('SMTP_STATUS_CONNECTED', 2, TRUE);

	class smtp{

		var $connection;
		var $recipients;
		var $headers;
		var $timeout;
		var $errors;
		var $status;
		var $body;
		var $from;
		var $host;
		var $port;
		var $helo;
		var $auth;
		var $user;
		var $pass;

		/***************************************
        ** Constructor function. Arguments:
		** $params - An assoc array of parameters:
		**
		**   host    - The hostname of the smtp server		Default: localhost
		**   port    - The port the smtp server runs on		Default: 25
		**   helo    - What to send as the HELO command		Default: localhost
		**             (typically the hostname of the
		**             machine this script runs on)
		**   auth    - Whether to use basic authentication	Default: FALSE
		**   user    - Username for authentication			Default: <blank>
		**   pass    - Password for authentication			Default: <blank>
		**   timeout - The timeout in seconds for the call	Default: 5
		**             to fsockopen()
        ***************************************/

		function smtp($params = array()){

			if(!defined('CRLF'))
				define('CRLF', "\r\n", TRUE);
			
			$this->timeout	= 5;
			$this->status	= SMTP_STATUS_NOT_CONNECTED;
			$this->host		= 'localhost';
			$this->port		= 25;
			$this->helo		= 'localhost';
			$this->auth		= FALSE;
			$this->user		= '';
			$this->pass		= '';
			$this->errors   = array();

			foreach($params as $key => $value){
				$this->$key = $value;
			}
		}

		/***************************************
        ** Connect function. This will, when called
		** statically, create a new smtp object, 
		** call the connect function (ie this function)
		** and return it. When not called statically,
		** it will connect to the server and send
		** the HELO command.
        ***************************************/

		function connect($params = array()){

			if(!isset($this->status)){
				$obj = new smtp($params);
				if($obj->connect()){
					$obj->status = SMTP_STATUS_CONNECTED;
				}

				return $obj;

			}else{
				$this->connection = fsockopen($this->host, $this->port, $errno, $errstr, $this->timeout);
				@socket_set_timeout($this->connection, 0, 250000);

				$greeting = $this->get_data();
				if(is_resource($this->connection)){
					return $this->auth ? $this->ehlo() : $this->helo();
				}else{
					$this->errors[] = 'Failed to connect to server: '.$errstr;
					return FALSE;
				}
			}
		}

		/***************************************
        ** Function which handles sending the mail.
		** Arguments:
		** $params	- Optional assoc array of parameters.
		**            Can contain:
		**              recipients - Indexed array of recipients
		**              from       - The from address. (used in MAIL FROM:),
		**                           this will be the return path
		**              headers    - Indexed array of headers, one header per array entry
		**              body       - The body of the email
		**            It can also contain any of the parameters from the connect()
		**            function
        ***************************************/

		function send($params = array()){

			foreach($params as $key => $value){
				$this->set($key, $value);
			}

			if($this->is_connected()){

				// Do we auth or not? Note the distinction between the auth variable and auth() function
				if($this->auth){
					if(!$this->auth())
						return FALSE;
				}

				$this->mail($this->from);
				if(is_array($this->recipients))
					foreach($this->recipients as $value)
						$this->rcpt($value);
				else
					$this->rcpt($this->recipients);

				if(!$this->data())
					return FALSE;

				// Transparency
				$headers = str_replace(CRLF.'.', CRLF.'..', trim(implode(CRLF, $this->headers)));
				$body    = str_replace(CRLF.'.', CRLF.'..', $this->body);
				$body    = $body[0] == '.' ? '.'.$body : $body;

				$this->send_data($headers);
				$this->send_data('');
				$this->send_data($body);
				$this->send_data('.');

				return (substr(trim($this->get_data()), 0, 3) === '250');
			}else{
				$this->errors[] = 'Not connected!';
				return FALSE;
			}
		}
		
		/***************************************
        ** Function to implement HELO cmd
        ***************************************/

		function helo(){
			if(is_resource($this->connection)
					AND $this->send_data('HELO '.$this->helo)
					AND substr(trim($error = $this->get_data()), 0, 3) === '250' ){

				return TRUE;

			}else{
				$this->errors[] = 'HELO command failed, output: ' . trim(substr(trim($error),3));
				return FALSE;
			}
		}
		
		/***************************************
        ** Function to implement EHLO cmd
        ***************************************/

		function ehlo(){
			if(is_resource($this->connection)
					AND $this->send_data('EHLO '.$this->helo)
					AND substr(trim($error = $this->get_data()), 0, 3) === '250' ){

				return TRUE;

			}else{
				$this->errors[] = 'EHLO command failed, output: ' . trim(substr(trim($error),3));
				return FALSE;
			}
		}
		
		/***************************************
        ** Function to implement AUTH cmd
        ***************************************/

		function auth(){
			if(is_resource($this->connection)
					AND $this->send_data('AUTH LOGIN')
					AND substr(trim($error = $this->get_data()), 0, 3) === '334'
					AND $this->send_data(base64_encode($this->user))			// Send username
					AND substr(trim($error = $this->get_data()),0,3) === '334'
					AND $this->send_data(base64_encode($this->pass))			// Send password
					AND substr(trim($error = $this->get_data()),0,3) === '235' ){

				return TRUE;

			}else{
				$this->errors[] = 'AUTH command failed: ' . trim(substr(trim($error),3));
				return FALSE;
			}
		}

		/***************************************
        ** Function that handles the MAIL FROM: cmd
        ***************************************/
		
		function mail($from){

			if($this->is_connected()
				AND $this->send_data('MAIL FROM:<'.$from.'>')
				AND substr(trim($this->get_data()), 0, 2) === '250' ){

				return TRUE;

			}else
				return FALSE;
		}

		/***************************************
        ** Function that handles the RCPT TO: cmd
        ***************************************/
		
		function rcpt($to){

			if($this->is_connected()
				AND $this->send_data('RCPT TO:<'.$to.'>')
				AND substr(trim($error = $this->get_data()), 0, 2) === '25' ){

				return TRUE;

			}else{
				$this->errors[] = trim(substr(trim($error), 3));
				return FALSE;
			}
		}

		/***************************************
        ** Function that sends the DATA cmd
        ***************************************/

		function data(){

			if($this->is_connected()
				AND $this->send_data('DATA')
				AND substr(trim($error = $this->get_data()), 0, 3) === '354' ){
 
				return TRUE;

			}else{
				$this->errors[] = trim(substr(trim($error), 3));
				return FALSE;
			}
		}

		/***************************************
        ** Function to determine if this object
		** is connected to the server or not.
        ***************************************/

		function is_connected(){

			return (is_resource($this->connection) AND ($this->status === SMTP_STATUS_CONNECTED));
		}

		/***************************************
        ** Function to send a bit of data
        ***************************************/

		function send_data($data){

			if(is_resource($this->connection)){
				return fwrite($this->connection, $data.CRLF, strlen($data)+2);
			}else
				return FALSE;
		}

		/***************************************
        ** Function to get data.
        ***************************************/

		function &get_data(){

			$return = '';
			$line   = '';

			if(is_resource($this->connection)){
				while((strpos($return, CRLF) == FALSE OR substr($line,3,1) != ' ') AND $count < 30){
					$line    = fgets($this->connection, 512);
					$return .= $line;
                    $count++;
          }
				return $return;

			}else
				return FALSE;
		}

		/***************************************
        ** Sets a variable
        ***************************************/
		
		function set($var, $value){

			$this->$var = $value;
			return TRUE;
		}

	} // End of class

/* 
 * Copyright (C) 1998, Manuel Lemos (mlemos@acm.org) 
 *  
 * Permission to use and modify this software and its 
 * documentation for any purpose other than its incorporation 
 * into a commercial product is hereby granted without fee, 
 * as long as the author is notified that this piece of software 
 * is being used in other applications. 
 * Permission to copy and distribute this software and its 
 * documentation only for non-commercial use is also granted 
 * without fee, provided, however, that the above copyright 
 * notice appear in all copies, that both that copyright notice 
 * and this permission notice appear in supporting documentation. 
 * The author makes no representations about the suitability 
 * of this software for any purpose.  It is provided ``as is'', 
 * without express or implied warranty. 
 */ 

class pop3_class 
{ 
 var $hostname= ""; 
 var $port=110; 

  /* Private variables - DO NOT ACCESS */ 

 var $connection=0; 
 var $state= "DISCONNECTED"; 
 var $greeting= ""; 
 var $must_update=0; 


  /* Private methods - DO NOT CALL */ 

 Function GetLine() 
 { 
  for($line= "";;) 
  { 
   if(@feof($this->connection)) 
    return(0); 
   $line.=fgets($this->connection,100); 
   $length=strlen($line); 
   if($length>=2 && substr($line,$length-2,2)== "\r\n") 
    return(substr($line,0,$length-2)); 
  } 
 } 

 Function PutLine($line) 
 { 
  return(fputs($this->connection, "$line\r\n")); 
 } 

 Function OpenConnection() 
 { 
  if($this->hostname== "") 
   return( "2 (Es wurde kein g&uuml;ltiger Servername angegeben)"); 
  switch(($this->connection=fsockopen($this->hostname,$this->port))) 
  { 
   case -3: 
    return("3 (Socket konnte nicht erzeugt werden)"); 
   case -4: 
    return("4 (DNS-Lookup f&uuml;r Host \"$hostname\" fehlgeschlagen)"); 
   case -5: 
    return("5 (Verbindung unterbrochen oder wegen Zeit&uuml;berschreitung beendet)"); 
   case -6: 
    return("6 (Aufruf von fdopen() fehlgeschlagen)"); 
   case -7: 
    return("7 (Aufruf von setvbuf() fehlgeschlagen)"); 
   default: 
    return(""); 
  } 
 } 

 Function CloseConnection() 
 { 
  if($this->connection!=0) 
  { 
   fclose($this->connection); 
   $this->connection=0; 
  } 
 } 

  /* Public methods */ 

  /* Open method - set the object variable $hostname to the POP3 server address. */ 

 Function Open() 
 { 
 
  if($this->state!= "DISCONNECTED") 
   return( "1 (Die Verbindung wurde bereits erfolgreich ge&ouml;ffnet)"); 
  if(($error=$this->OpenConnection())!= "") 
   return($error); 
  $this->greeting=$this->GetLine(); 
  if(GetType($this->greeting)!= "string" 
  || strtok($this->greeting, " ")!= "+OK") 
  { 
   $this->CloseConnection(); 
   return( "3 (Der POP3 Server hat keine Begr&uuml;&szlig;ung &uuml;bermittelt)"); 
  } 
  $this->greeting=strtok( "\r\n"); 
  $this->must_update=0; 
  $this->state= "AUTHORIZATION"; 
 
  return( ""); 
 } 
  
  /* Close method - this method must be called at least if there are any 
     messages to be deleted */ 

 Function Close() 
 { 
  if($this->state== "DISCONNECTED") 
   return( "Es wurde keine Verbindung ge&ouml;ffnet"); 
  if($this->must_update) 
  { 
   if($this->PutLine( "QUIT")==0) 
    return( "Das <code>QUIT</code>-Kommando konnt nicht gesendet werden"); 
   $response=$this->GetLine(); 
   if(GetType($response)!= "string") 
    return( "Konnte die Quittung des letzten Kommandos nicht empfangen"); 
   if(strtok($response, " ")!= "+OK") 
    return( "Konnte die folgende Verbindung nicht quittieren: ".strtok("\r\n")); 
  } 
  $this->CloseConnection(); 
  $this->state= "DISCONNECTED"; 
  return( ""); 
 } 

  /* Login method - pass the user name and password of POP account.  Set 
     $apop to 1 or 0 wether you want to login using APOP method or not.  */ 

 Function Login($user,$password,$apop)  { 
       if($this->state!= "AUTHORIZATION") return( "Verbindung ist nicht im <code>AUTHORIZATION</code>-Status"); 
      if($apop) { 
        if($this->PutLine( "APOP $user ".md5($this->greeting.$password))==0) return( "Konnte das <code>APOP</code>-Kommando nicht senden"); 
        $response=$this->GetLine(); 
        if(GetType($response)!= "string") return( "Habe keine Antwort auf <code>APOP</code>-Login empfangen"); 
        if(strtok($response, " ")!= "+OK") return( "<code>APOP</code>-Login fehlgeschlagen: ".strtok( "\r\n")); 
    }  else  { 
        if($this->PutLine( "USER $user")==0) return( "Konnte das <code>USER</code>-Kommando nicht senden"); 
        $response=$this->GetLine(); 
        if(GetType($response)!= "string") return( "Konnte keine Antwort auf User-Login empfangen"); 
        if(strtok($response, " ")!= "+OK") return( "<code>USER</code>-Fehler: ".strtok( "\r\n")); 
        if($this->PutLine( "PASS $password")==0) return( "Konnte das <code>PASS</code>-Kommando nicht senden"); 
        $response=$this->GetLine(); 
        if(GetType($response)!= "string") return( "Konnte auf das <code>PASS</code>-Kommando keine Antwort empfangen"); 
        if(strtok($response, " ")!= "+OK") return( "Kennwort-Fehler: ".strtok( "\r\n")); 
    } 
    $this->state= "TRANSACTION"; 
 
      return( ""); 
 } 

  /* Statistics method - pass references to variables to hold the number of 
     messages in the mail box and the size that they take in bytes.  */ 

 }; 

/* ---- pop3.php3 class file ends here. ---- */ 

?>
